#!/bin/sh -e
#
# SYPNOSIS
#   patch-script.sh [--apply|--unpatch] KERNEL_DIR PATCHNAME
#   /usr/src/kernel-patches/ARCH/[apply|unpatch]/PATCHNAME
#
# DESCRIPTION
#
#   Apply / Remove a patch to the kernel.  These input files direct its
#   actions:
#
#       PATCHNAME-patches.tar.gz
#	    Contains the different patches for each kernel version.
#	    The files in the archive must extract to the current
#	    directory.
#
#       PATCHNAME-versions.txt (1st form)
#	    An option file with lines of the format:
#	        KERNEL_VERSION patchfilename ...
#	    KERNEL_VERSION can have shell wildcards.  patchfilename is
#	    the name of a file in PATCHNAME-patches.tar.gz.  A "." means
#           "do not patch this version".  An example:
#               2.2.*  .
#	        2.4.*  usbeth-2.4-0.patch
#	        2.6.*  usbeth-2.6-0.patch
#
#   In the first form, the data files PATCHNAME-patches.tar.gz and
#   PATCHNAME-versions.txt must live in current directory.  The
#   kernel source directory is passed on the command line.  In the
#   second form, the command must be run in the kernel source
#   directory, and the input files must be found in:
#
#	/usr/src/kernel-patches/ARCH/PATCHNAME
#
#   The KERNEL_VERSION is read from the kernel Makefile.  It is
#   $VERSION.$PATCHLEVEL.$SUBLEVEL$EXTRAVERSION.  The patches applied
#   are chosen as follows:
#
#	1.  PATCHNAME-versions.txt is searched.  The first matching line
#	    is used.
#
#       2.  If no matching line is found, all patches whose names match
#	    PATCHNAME-KERNEL_VERSION-*.patch are applied.
#
#   If no applicable patch is found the scripts exits with an error.
#   Patches are sorted (sort -u) by name before being applied.
#
#
# NOTES
#
#   This is an alternative to dh_installkpatches.  That script has
#   caused me one too many problems.  This is a home brew replacement.
#

#
# Process command line arguments.
#
case "$1" in
  --apply|--unpatch)
      action=`echo "$1" | sed 's/..//'`
      kernelDir="$1"
      patchName="$2"
      patchDir=.
      ;;
  '')		
      me=`which $0`
      patchName=`basename $me`
      action=`dirname $me`
      patchDir=`dirname $action`/$patchName
      action=`basename $action`
      kernelDir=.
      ;;
  *)
    echo 1>&2 "usage: $0, OR"
    echo 1>&2 "       $0 [--apply|--unpatch] kernel_source patchName"
    exit 1
esac

#
# Grab the kernel version from the kernel Makefile.
#
getVer() {
  sed -n "s/^$1[	 ]*=[ 	]*\([^ 	]*\).*/\1/p" "$kernelDir/Makefile" 2>/dev/null
}

KERNEL_VERSION=`getVer VERSION`.`getVer PATCHLEVEL`.`getVer SUBLEVEL``getVer EXTRAVERSION`

case "$KERNEL_VERSION" in
  2.[46].[1-9]*) ;;
  *)
        echo 1>&2 "$0: $kernelDir doesn't look like a kernel source directory!"
        exit 1
esac

#
# Get some file names.
#
patchFile="$patchDir/$patchName-patches.tar.gz"
versionFile="$patchDir/$patchName-versions.txt"
tmphome="debian/$patchName-patchtmp"
tmpdir="$kernelDir/$tmphome"

[ -s "$patchFile" ] || {
    echo 1>&2 "$0: Can't find my patch archive $patchFile.  Aborting ..."
    exit 1
  }

#
# Unpack the patches into a temporary directory.
#
rm -rf "$tmpdir"
mkdir -p "$tmpdir"
patchlist=`tar --extract --gzip --directory "$tmpdir" --file "$patchFile" --verbose`

#
# Determine which patches to apply.
#
[ ! -s "$versionFile" ] || {
  parseVersions() {
    while read pattern patchline
    do
      case "$KERNEL_VERSION" in
	$pattern)	echo "$patchline"
			break ;;
      esac
    done <"$versionFile"
  }
  patches=`parseVersions`
}

#
# A Patch of . means there is nothing more to do.
#
[ ."$patches" != ."." ] ||
  exit 0

[ -n "$patches" ] ||
  patches=`echo "$patchlist" | grep "$patchName-$KERNEL_VERSION-.*\.patch"` || :

#
# Figure out which direction we are going
#
case "$action" in
  apply)	reverse= ;;
  unpatch)	reverse=--reverse ;;
esac

patches=`echo $patches | sort --unique $reverse`

[ -n "$patches" ] || {
    echo 1>&2 "$0: I don't have a patch for this kernel.  Aborting ..."
    exit 1
  }

#
# Do a test run, to try and figure out if this will all work
# before corrupting his sources.
#
for patch in $patches
do
  p=$tmphome/$patch
  patch --dry-run --silent --force --strip=1 --input="$p" --directory=$kernelDir $reverse || {
      patch --dry-run --force --strip=1 --input="$p" --directory=$kernelDir $reverse || {
	echo 1>&2 "$0: Patch $p won't $action cleanly. Aborting ..."
	exit 1
      }
    }
done

#
# It seems to work, baring conflicting changes within this one patch.
# Patch everything for real.
#
for patch in $patches
do
  p=$tmphome/$patch
  patch --silent --force --strip=1 --input="$p" --directory=$kernelDir $reverse || {
      echo 1>&2 "$0: Patch $p didn't $action cleanly. Your sources are probably corrupted."
      echo 1>&2 -n "Press ENTER to abort: "
      read x
      exit 1
    }
done

#
# Clean up.
#
rm -r "$tmpdir"
